package strangepatterns2.objectadaptation;

import java.util.Map;
import java.util.HashMap;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */

public class AdapterRegistry {
//
  private static final AdapterRegistry instance = new AdapterRegistry();
//
  private static Map adapters = new HashMap();

  private AdapterRegistry() {
  }

  public static AdapterRegistry getInstance() {
    return instance;
  }

  public void registerAdapter(Class source, Class target, Class adapterType) {
    adapters.put(source.getName() + "_" + target.getName(), adapterType);
  }

  public Object getAdapter(Object source, Class target) {
    if(source==null) return null;
    if(target.isInstance(source)) return source;
    Class adapterType = (Class) adapters.get(source.getClass().getName() + "_" + target.getName());
    if (adapterType == null) {
     adapterType = findBySuperClass(source,target);
    }
    if (adapterType == null) {
     adapterType = findByInterfaces(source,target);
    }
    if(adapterType!=null){
     return newInstance(adapterType);
    }
   return null;
  }

  private Class findBySuperClass(Object source, Class target){
    Class parent = source.getClass().getSuperclass();
    Class adapter = null;
    while(parent != null){
     adapter = (Class) adapters.get(parent.getName() + "_" + target.getName());
     if(adapter != null) return adapter;
     parent = parent.getSuperclass();
    }
   return adapter;
  }

  private Class findByInterfaces(Object source, Class target){
    Class adapter = null;
    Class [] interfaces = source.getClass().getInterfaces();

    for (int i = 0; i < interfaces.length; i++) {
      adapter = (Class) adapters.get(interfaces[i].getName() + "_" + target.getName());
     if(adapter != null) return adapter;
    }
   return adapter;
  }

  private Object newInstance(Class adapterType){
    try {
        return adapterType.newInstance();
      }
      catch (Exception ex) {
        ex.printStackTrace();
        return null;
      }
  }

} //END